/* jshint undef: true */
/* global plugin, require, com, java */

var _ = require("lodash");
var on = require("on")(plugin);
var Objects = require("convenience").Objects;
var facade = require("facade");
var request = require("request");
var SystemInformationUtilities = com.cattura.agents.configuration_module.model.utilities.SystemInformationUtilities;
var StringUtilities = com.cattura.cattura_commons.utilities.StringUtilities;
var EventType = com.cattura.agents.configuration_module.model.vos.events.EventType;
var STORE = plugin.info().dataStore;
var settings = plugin.info().settings;
var LOGGER = plugin.info().logger;

function Notifier (data)
{
    _(this).extend(data).value();
    
    this.canHandleEvent = function canHandleEvent (event)
    {
        if (event.type.category == EventType.Category.INTERNAL)
        {
            return false;
        }
        
        return _(this.events).find(function (type) { 
            return type == event.type.name() || type == event.getClass().simpleName;
        });
    };
    
    this.handleEvent = function handleEvent (event)
    {
        request.post(this.url, {
            form: {
                eventSource: getUnitName(),
                eventType: event.type.name(),
                eventTime: event.time.time,
                eventDescription: event.description,
                eventData: JSON.stringify(Objects.toJS(event.data, true))
            }
        }, function (err, response, body) {
            if (err)
            {
                LOGGER.warning("Error: " + JSON.stringify(err));
            }
        });
    };
}

function getNotifiers ()
{
    return _(Objects.toJS(STORE.select("notifier:*"))).map(function (notifier) {
        return new Notifier(notifier);
    }).value();
}

function addNotifier (id, url, events)
{
    if (!(_(url).startsWith("http://") || _(url).startsWith("https://")))
    {
        url = "http://" + url;
    }
    
    STORE.set("notifier:" + id, {id: id, url: url, events: events});
}

function removeNotifier (id)
{
    STORE.remove("notifier:" + id);
}

function getNotifiersForEvent (event)
{
    return _(getNotifiers()).filter(function (notifier) {
        return notifier.canHandleEvent(event);
    }).value();
}

function generateIdForNotifier ()
{
    var id = StringUtilities.createRandomString(32, "abcdefghijklmnopqrstuvwxyz0123456789");
    if (_(getNotifiers()).find(function (notifier) { return notifier.id === id; }))
    {
        return generateIdForNotifier();
    }
    
    return id;
}

function getUnitName ()
{
    var settings = facade.retrieveProxy("SettingsProxy");
    var unitName = settings.getUserSettings().getString("unit-name").or("");
    if (!unitName.length())
    {
        unitName = SystemInformationUtilities.getMachineName();
    }
    return unitName;
}

function getExceptionFromEvent (event)
{
    if ('incident' in event)
    {
        return event.incident.exception;
    }
    else if ('exception' in event)
    {
        return event.exception;
    }
}

plugin.service({
    name: "addNotifier",
    description: "Add a notifier",
    parameters: [
        {name: "url", description: "URL", type: "java.lang.String"},
        {name: "events", description: "Events to notify on", type: "java.util.List"}
    ],
    handler: function (url, events) {
        addNotifier(generateIdForNotifier(), url, events);
    }
});

plugin.service({
    name: "addNotifier",
    description: "Add a notifier",
    parameters: [
        {name: "id", description: "Unique identifier for notifier", type: "java.lang.String"},
        {name: "url", description: "URL", type: "java.lang.String"},
        {name: "events", description: "Events to notify on", type: "java.util.List"}
    ],
    handler: addNotifier
});

plugin.service({
    name: "removeNotifier",
    description: "Remove a notifier",
    parameters: [
        {name: "id", description: "Unique identifier for notifier", type: "java.lang.String"}
    ],
    handler: removeNotifier
});

plugin.service({
    name: "getNotifiers",
    description: "Return all registered notifiers",
    parameters: [],
    handler: getNotifiers
});

plugin.onEvent(function(event) {
    try
    {
        _(getNotifiersForEvent(event)).each(function (notifier) {
            notifier.handleEvent(event);
        });
    }
    catch (exception)
    {
        if (getExceptionFromEvent(event))
        {
            LOGGER.info("Webhook Plugin encountered (and swallowed) exception while trying to handle exception event");
            return;
        }
        
        throw new java.lang.RuntimeException("Webhook Plugin encountered exception while trying to handle event: " + event, exception.javaException);
    }
});
